#!/usr/bin/env python3

from typing import Union

import aiohttp

from gv_utils import parser
from gv_utils.datetime import datetime, to_str
from gv_utils.enums import RequestParam, RequestPath
from gv_utils.geometry import BaseGeometry, encode_geometry
from gv_utils.logger import Logger

SEPARATOR = RequestPath.separator


class Requests:

    def __init__(self, logger: Logger) -> None:
        self.logger = logger
        self.apiurl = None

    async def async_init(self, apiaddr):
        self.apiurl = 'http://{}/api'.format(apiaddr)

    async def get_data_point(self, datapointeids: list = None, datatypeeids: list = None,
                             area: BaseGeometry = None) -> list:
        params = {}
        if datatypeeids is not None:
            params[RequestParam.datatype] = self.__encode_list_params(datatypeeids)
        return await self._get_location(datapointeids, area, RequestPath.datapoint, params)

    async def get_road(self, roadeids: list = None, area: BaseGeometry = None) -> list:
        return await self._get_location(roadeids, area, RequestPath.road)

    async def get_road_center(self, roadeids: list = None, area: BaseGeometry = None) -> list:
        return await self._get_location(roadeids, area, SEPARATOR.join((RequestPath.road, RequestPath.roadcenter)))

    async def get_turn_ratio(self, fromroad: str = None, toroad: str = None) -> list:
        params = {}
        if fromroad is not None:
            params[RequestParam.fromroad] = fromroad
        if toroad is not None:
            params[RequestParam.toroad] = toroad
        return await self._get(SEPARATOR.join((RequestPath.road, RequestPath.roadcenter, RequestPath.turnratio)),
                               params)

    async def get_prediction_point(self) -> list:
        return await self._get(RequestPath.predictionpoint)

    async def get_zone_point(self, zoneeids: list = None, area: BaseGeometry = None) -> list:
        return await self._get_location(zoneeids, area, RequestPath.zonepoint)

    async def _get_location(self, eids: list, area: BaseGeometry, applyto: str, params: dict = None) -> list:
        if params is None:
            params = {}

        if eids is not None:
            params[RequestParam.eid] = self.__encode_list_params(eids)
        if area is not None:
            params[RequestParam.within] = encode_geometry(area)
        return await self._get(applyto, params)

    async def get_road_data_point(self, datatype: str = None, area: BaseGeometry = None) -> list:
        params = {}
        if area is not None:
            params[RequestParam.within] = encode_geometry(area)
        if datatype is not None:
            params[RequestParam.datatype] = datatype
        return await self._get(SEPARATOR.join((RequestPath.road, RequestPath.datapoint)), params)

    async def get_data_point_data_quality(self, fromdatetime: datetime, todatetime: datetime = None,
                                          datapointeids: list = None, datatypeeids: list = None, period: str = None,
                                          sampling: str = None, window: str = None) -> dict:
        params = {}
        if datatypeeids is not None:
            params[RequestParam.datatype] = self.__encode_list_params(datatypeeids)
        return await self._get_data_quality(fromdatetime, todatetime, datapointeids, period, sampling, window,
                                            RequestPath.datapoint, params)

    async def get_data_type_data_quality(self, fromdatetime: datetime, todatetime: datetime = None,
                                         datatypeeids: list = None, period: str = None, sampling: str = None,
                                         window: str = None) -> dict:
        return await self._get_data_quality(fromdatetime, todatetime, datatypeeids, period, sampling, window,
                                            RequestPath.datatype)

    async def get_road_data_quality(self, fromdatetime: datetime, todatetime: datetime = None, roadeids: list = None,
                                    period: str = None, sampling: str = None, window: str = None) -> dict:
        return await self._get_data_quality(fromdatetime, todatetime, roadeids, period, sampling, window,
                                            RequestPath.road)

    async def _get_data_quality(self, fromdatetime: datetime, todatetime: datetime, locationeids: list,
                                period: str, sampling: str, window: str, applyto: str, params: dict = None) -> dict:
        return await self.__get_data(fromdatetime, todatetime, locationeids, period, sampling, window,
                                     SEPARATOR.join((applyto, RequestPath.dataquality)), params)

    async def get_data_point_indicator(self, indicator: str, fromdatetime: datetime,
                                       todatetime: datetime = None, datapointeids: list = None,
                                       datatypeeids: list = None, period: str = None, sampling: str = None,
                                       window: str = None) -> dict:
        params = {RequestParam.indicator: indicator}
        if datatypeeids is not None:
            params[RequestParam.datatype] = self.__encode_list_params(datatypeeids)
        return await self.__get_data(fromdatetime, todatetime, datapointeids, period, sampling, window,
                                     SEPARATOR.join((RequestPath.datapoint, RequestPath.indicator)), params)

    async def get_road_indicator(self, indicator: str, fromdatetime: datetime, todatetime: datetime,
                                 roadeids: list = None, period: str = None, sampling: str = None,
                                 window: str = None) -> dict:
        params = {RequestParam.indicator: indicator}
        return await self.__get_data(fromdatetime, todatetime, roadeids, period, sampling, window,
                                     SEPARATOR.join((RequestPath.road, RequestPath.indicator, indicator)), params)

    async def __get_data(self, fromdatetime: datetime, todatetime: datetime, locationeids: list, period: str,
                         sampling: str, window: str, applyto: str, params: dict) -> dict:
        if params is None:
            params = {}

        params[RequestParam.fromdatetime] = to_str(fromdatetime)
        if todatetime is not None:
            params[RequestParam.todatetime] = to_str(todatetime)
        if locationeids is not None:
            params[RequestParam.eid] = self.__encode_list_params(locationeids)
        if period is not None:
            params[RequestParam.period] = period
        if sampling is not None:
            params[RequestParam.sampling] = sampling
        if window is not None:
            params[RequestParam.window] = window
        return await self._get(applyto, params)

    async def get_zone_travel_time(self, fromdatetime: datetime, todatetime: datetime, period: str, sampling: str,
                                   frompointeid: str = None, topointeid: str = None) -> dict:
        params = {
            RequestParam.fromdatetime: to_str(fromdatetime)
        }
        if todatetime is not None:
            params[RequestParam.todatetime] = to_str(todatetime)
        if period is not None:
            params[RequestParam.period] = period
        if sampling is not None:
            params[RequestParam.sampling] = sampling
        if frompointeid is not None:
            params[RequestParam.frompoint] = frompointeid
        if topointeid is not None:
            params[RequestParam.topoint] = topointeid
        return await self._get(SEPARATOR.join((RequestPath.zonepoint, RequestPath.traveltime)), params)

    async def _get(self, suffix: str, params: dict = None) -> Union[list, dict]:
        if params is None:
            params = {}
        params[RequestParam.internalflag] = ''
        message = []
        async with aiohttp.ClientSession() as session:
            url = '{}{}'.format(SEPARATOR.join((self.apiurl, suffix)), SEPARATOR)
            async with session.get(url, params=params) as resp:
                if resp.status == 200:
                    message = await parser.decode_message(await resp.text())
        return message

    @staticmethod
    def __encode_list_params(params: list) -> str:
        return RequestParam.separator.join(params)

    async def close(self) -> None:
        pass



#!/usr/bin/env python3


class DbColumn:
    separator = '.'

    att = 'att'
    confidence = 'confidence'
    datapoint = 'data_point'
    datatype = 'data_type'
    dataquality = 'data_quality'
    day = 'day'
    density = 'density'
    eid = 'eid'
    entrance = 'entrance'
    flow = 'flow'
    flowin = 'flow_in'
    flowout = 'flow_out'
    fluidity = 'fluidity'
    fromtraveltime = 'from_travel_time'
    fromroad = 'from_road'
    fromturnratio = 'from_turn_ratio'
    fromzonepoint = 'from_zone_point'
    geom = 'geom'
    id = 'id'
    indicator = 'indicator'
    nbike = 'nbike'
    nbus = 'nbus'
    ncar = 'ncar'
    networkdataquality = 'network_data_quality'
    networkindicator = 'network_indicator'
    occupancy = 'occupancy'
    prediction = 'prediction'
    predictionpoint = 'prediction_point'
    ratio = 'ratio'
    road = 'road'
    roadcenter = 'road_center'
    samples = 'samples'
    sampletime = 'sample_time'
    timestamp = 'timestamp'
    speed = 'speed'
    totraveltime = 'to_travel_time'
    toroad = 'to_road'
    toturnratio = 'to_turn_ratio'
    tozonepoint = 'to_zone_point'
    traveltime = 'travel_time'
    validfrom = 'valid_from'
    validto = 'valid_to'
    webatt = 'web_att'


class DbTable:
    datapoint = 'data_point'
    datapointdataquality = 'data_point_data_quality'
    datapointindicator = 'data_point_indicator'
    datatype = 'data_type'
    networkdataquality = 'network_data_quality'
    networkindicator = 'network_indicator'
    prediction = 'prediction'
    predictionpoint = 'prediction_point'
    road = 'road'
    roadcenter = 'road_center'
    roaddatapoint = 'road_data_point'
    roaddataquality = 'road_data_quality'
    roaddensity = 'road_density'
    roadfluidity = 'road_fluidity'
    roadpartition = 'road_partition'
    roadvehicle = 'road_vehicle'
    turnratio = 'turn_ratio'
    zonepoint = 'zone_point'
    zonepointtraveltime = 'zone_point_travel_time'


class Message:
    data = 'data'
    format = 'format'
    timestamp = 'timestamp'
    kind = 'kind'


class MessageData(DbColumn):
    csvseparator = ';'
    csvquote = '\''
    encoding = 'utf-8'
    separator = '_'

    status = 'status'

    late = -2
    nan = -1

    hendday = 21
    hstartday = 6

    avg = 'avg'
    max = 'max'
    min = 'min'
    raw = 'raw'


class AttId(DbColumn):
    centerxy = 'centerxy'
    centroid = 'centroid'
    datapointseids = 'datapoints'
    datatypeeid = 'datatypeeid'
    ffspeed = 'ffspeed'
    fow = 'fow'
    frc = 'frc'
    fromno = 'fromno'
    geomxy = 'geomxy'
    length = 'length'
    mainroad = 'mainroad'
    maxspeed = 'maxspeed'
    name = 'name'
    nlanes = 'nlanes'
    no = 'no'
    roads = 'roads'
    tono = 'tono'
    x = 'x'
    y = 'y'
    zone = 'zone'


class MessageFormat:
    json = 'json'
    csv = 'csv'


class MessageKind:
    separator = '-'

    mainprefix = 'gtlv'

    past = 'past'

    dataquality = 'dataquality'
    datapoint = 'datapoint'
    datatype = 'datatype'
    indicator = 'indicator'
    mappingroadsdatapoints = 'mappingroadsdatapoints'
    network = 'network'
    partitions = 'partitions'
    prediction = 'prediction'
    predictionpoint = 'predictionpoint'
    traveltime = 'traveltime'
    turnratio = 'turnratio'
    vehicles = 'vehicles'

    density = 'density'
    fluidity = 'fluidity'
    karrusrd = 'karrusrd'
    metropme = 'metropme'
    road = 'road'
    roadcenter = 'roadcenter'
    tomtomfcd = 'tomtomfcd'
    zonepoint = 'zonepoint'


class RequestParam(MessageKind):
    separator = ','

    internalflag = 'internal'

    eid = 'eid'
    fields = 'fields'
    fromdatetime = 'fromdatetime'
    frompoint = 'frompoint'
    fromroad = 'fromroad'
    period = 'period'
    sampling = 'sampling'
    todatetime = 'todatetime'
    topoint = 'topoint'
    toroad = 'toroad'
    window = 'window'
    within = 'within'

    day = 'day'
    month = 'month'
    week = 'week'


class RequestPath(MessageKind):
    separator = '/'

    confidence = 'confidence'
    flow = 'flow'
    occupancy = 'occupancy'
    speed = 'speed'


class NetworkObjId:
    datapointsroadsmap = 'datapointsroadsmap'
    frcroadsmap = 'frcroadsmap'
    lonlatnodesmatrix = 'lonlatnodesmatrix'
    mainclustersgeom = 'mainclustersgeom'
    newdpmappings = 'newdpmappings'
    newzonemappings = 'newzonemappings'
    omiteddatapoints = 'omiteddatapoints'
    roadclustermap = 'roadclustermap'
    roadsdatamap = 'roadsdatamap'
    roadsffspeedmap = 'roadsffspeedmap'
    roadszonesmap = 'roadszonesmap'
    voronoiroadmap = 'voronoiroadmap'
    zonesdatamap = 'zonesdatamap'



#!/usr/bin/env python3

import os

from dotenv import load_dotenv


def load_env_var(filepath: str) -> (str, str):
    projectroot = os.path.dirname(os.path.dirname(os.path.realpath(filepath)))

    envloaded = os.environ.get('ENV_LOADED', False)
    if not envloaded:
        dotenv_path = os.path.join(projectroot, '.env')
        load_dotenv(dotenv_path)
        envtype = os.environ.get('ENV_TYPE', default='local')
        load_dotenv(dotenv_path + '.' + envtype)
    else:
        envtype = os.environ.get('ENV_TYPE', default='local')

    return projectroot, envtype
